/**
 * $Id: Fortran77CodeEditor.java,v 1.3 2009/07/28 22:55:08 juanca Exp $
 * editors.* is part of ACodeEditor
 * Copyright (C) 2009 Juan Carlos Rodrguez-del-Pino. All rights reserved.
 * license GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author Juan Carlos Rodriguez-del-Pino
 **/
package editors;

import java.util.HashSet;
import java.util.Set;

import javax.swing.text.MutableAttributeSet;

public class Fortran77CodeEditor extends GenericCodeEditor {
	private static final long serialVersionUID = -2404070789317333160L;
	enum States {
		REGULAR, IN_LINE_COMMENT, IN_STRING, IN_DSTRING, IN_CSTRING
	};

	protected Set<String> declarators = new HashSet<String>();

	protected void initializeReserved() {
		{
			String list[] = {
				"ACCEPT", "ASSIGN",	"BACKSPACE",
				"CALL",	"CLOSE", "CONTINUE",
				"DECODE", "DO",	"DOWHILE",
				"ELSE", "ELSEIF", "ENCODE",
				"ENDDO", "ENDFILE",	"ENDIF",
				"GOTO",	"IF", "INCLUDE",
				"INQUIRE", "OPEN", "PAUSE",
				"PRINT", "RETURN", "REWIND",
				"SAVE", "STATIC", "STOP",
				"WRITE"};
			reserved.clear();
			for (int i = 0; i < list.length; i++)
				reserved.add(list[i]);
		}
		{
			String list[] = { "AUTOMATIC", "BLOCKDATA", "BYTE", "CHARACTER",
					"COMMON", "COMPLEX", "DATA", "DIMENSION", "DOUBLECOMPLEX",
					"DOUBLEPRECISION", "END", "ENDMAP", "ENDSTRUCTURE",					"ENDUNION",
					"EQUIVALENCE", "EXTERNAL", "FORMAT", "FUNCTION",
					"IMPLICIT",	"INTEGER", "INTRINSIC", "LOGICAL",
					"MAP", "NAMELIST", "OPTIONS", "PARAMETER",
					"POINTER", "PRAGMA", "PROGRAM", "REAL",
					"RECORD", "STATIC", "STRUCTURE", "SUBROUTINE",
					"TYPE", "UNION", "VIRTUAL", "VOLATILE" };
			declarators.clear();
			for (int i = 0; i < list.length; i++)
				declarators.add(list[i]);
		}
	}

	public Fortran77CodeEditor(Highlights fonts) {
		super(fonts);
		initializeReserved();
	}
//TODO Redo FORTRAN HightLighter
	protected boolean nextIsOpenParenthesis(String s, int begin) {
		for (int i = begin; i < s.length(); i++) {
			char c = s.charAt(i);
			if (c == '(')
				return true;
			if (c != ' ' && c != CR && c != LF && c != '\t')
				return false;
		}
		return false;
	}

	protected boolean isIdentifierChar(char c) {
		return (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z')
				|| (c >= '0' && c <= '9') || (c == '$') || (c == '_');
	}

	protected void showPending(String text, int begin, int end) {
		if (begin > end || end >= text.length())
			return;
		MutableAttributeSet type = font.getRegular();
		if (text.charAt(begin) == '$') { // Variable
			type = font.getVariable();
		} else if (isIdentifier(text, begin, end)) {
			String pendiente = text.substring(begin, end + 1);
			if (reserved.contains(pendiente))
				type = font.getReserved();
			else if (declarators.contains(pendiente))
				type = font.getVariable();
		}
		highlight(begin, end, type);
	}

	public void codeHighlight() {
		States state = States.REGULAR;
		String s = getText();
		int l = s.length();
		int beginPending = 0;
		for (int i = 0; i < l; i++) {
			if ((i % 100 == 0) && getNeedUpdate()) {
				return;
			}
			char current = s.charAt(i);
			char next;
			if (i + 1 < l)
				next = s.charAt(i + 1);
			else
				next = '\0';
			if (current == CR) // Normalizamos los finales de linea
				if (next == LF)
					continue;
				else
					current = LF;
			switch (state) {
			case REGULAR: { // Situacin normal
				if (current == '#') {// Comienza un comentario
					showPending(s, beginPending, i - 1);
					beginPending = i;
					state = States.IN_LINE_COMMENT;
				} else if (current == '"') {
					showPending(s, beginPending, i - 1);
					beginPending = i;
					state = States.IN_DSTRING;
				} else if (current == '\\') {
					i++;
				} else if (current == '\'') {
					showPending(s, beginPending, i - 1);
					beginPending = i;
					state = States.IN_STRING;
				} else if (current == '$') {
					if (next == '\'') {
						showPending(s, beginPending, i - 1);
						beginPending = i;
						i++;
						state = States.IN_CSTRING;
					} else if ((next >= '0' && next <= '9') || next == '*'
							|| next == '@' || next == '#' || next == '?'
							|| next == '-' || next == '$' || next == '!'
							|| next == '_') { // Parametros
						showPending(s, beginPending, i - 1);
						beginPending = i;
						showPending(s, i, i + 1);
						beginPending = i + 2;
						i++;
					} else {
						showPending(s, beginPending, i - 1);
						beginPending = i;
					}
				} else if (isIdentifierChar(current)) {
					if (beginPending < i
							&& !isIdentifierChar(s.charAt(beginPending))) {
						showPending(s, beginPending, i - 1);
						beginPending = i;
					}
				} else {
					if (beginPending < i
							&& isIdentifierChar(s.charAt(beginPending))) {
						showPending(s, beginPending, i - 1);
						beginPending = i;
					}
				}
			}
				break;
			case IN_LINE_COMMENT: {
				if (current == LF) {
					highlight(beginPending, i, font.getComment());
					beginPending = i + 1;
					state = States.REGULAR;
				}
			}
				break;
			case IN_CSTRING:
				if (current == '\'') {
					highlight(beginPending, i, font.getString());
					beginPending = i + 1;
					state = States.REGULAR;
				} else if (current == '\\') {
					i++;
				}
				break;
			case IN_DSTRING: {
				if (current == '"') {
					if (s.charAt(beginPending) == '$') {
						highlight(beginPending, i - 1, font.getVariable());
						beginPending = i;
					}
					highlight(beginPending, i, font.getString());
					beginPending = i + 1;
					state = States.REGULAR;
				} else if (current == '\\') {
					i++;
				} else if (current == '$') {
					if (beginPending < i) {
						if (s.charAt(beginPending) == '$')
							highlight(beginPending, i - 1, font.getVariable());
						else
							highlight(beginPending, i - 1, font.getString());
						beginPending = i;
					}
				} else {
					if (beginPending < i && !isIdentifierChar(current)) {
						if (s.charAt(beginPending) == '$')
							highlight(beginPending, i - 1, font.getVariable());
						else
							highlight(beginPending, i - 1, font.getString());
						beginPending = i;
					}
				}
			}
				break;
			case IN_STRING: {
				if (current == '\'') {
					highlight(beginPending, i, font.getString());
					beginPending = i + 1;
					state = States.REGULAR;
				}
			}
				break;
			}
		}
		switch (state) {
		case REGULAR:
			showPending(s, beginPending, l - 1);
			break;
		case IN_DSTRING:
		case IN_CSTRING:
		case IN_STRING:
			highlight(beginPending, l - 1, font.getString());
			break;

		case IN_LINE_COMMENT:
			highlight(beginPending, l - 1, font.getComment());
			break;
		}
		endUpdate();
	}

	/*	  


	   void MuestraCodigoFortran77::muestraTexto(HTML &pagina,const string &nf, const string &s){
	      inicializa();
	      nombreFichero=nf;
	      string fuente=s;
	      unsigned int pos=0;
	      enRistra=false;
	      while(pos<fuente.size()) //Se muestra lnea a lnea
	      {
	         string linea=extraeLinea(fuente,pos);
		 if(pos>0) pagina << finParrafo;
	         avanzaNumeroLinea(pagina);
		 if(linea.size() == 0) continue;
		 char primerCaracter=linea[0];
		 //Comprobar tipo de lnea que es
		 //Mostrar lnea comentario
		 if(primerCaracter == 'C' || primerCaracter=='c'
		     || primerCaracter == '*' || primerCaracter == '!') {
		     pagina << colorComentario << linea << finColorComentario; 
		 }
		 //Mostrar lnea depurado
		 else if(primerCaracter == 'D' || primerCaracter=='d') {
		     pagina << colorMacro << linea << finColorMacro; 
		 }
		 //Mostrar lnea macro
		 else {
		    //Ajuste tabs
		    for(unsigned int i=0; i <linea.size(); i++)
		       if(linea[i]=='\t') linea.replace(i,1,"        ");
		    if(linea.size()<7) { //Si pequea mostramos y seguimos
		      pagina << linea;
		      continue;
		    }
		    //mostramos nmeros
		    pagina << colorRistra << linea.substr(0,6) << finColorRistra;
		    char ccont=linea[5];
		    linea.erase(0,6);
		    if(ccont != ' '){
		       muestraRestoLinea(pagina,linea.substr(6,linea.size()-6)); //Lnea continuacin 
		       continue;
		    }
		    enRistra=false;
		    if(linea.size()>150) linea.erase(150,linea.size()-150);
		    //eliminacin de espacios
		    unsigned int posReal[151];
		    string lineaSE;
		    for(unsigned int i=0; i <linea.size(); i++)
		       if(linea[i]!=' ') {
		       posReal[lineaSE.size()]=i;
		       lineaSE+=alfabeto.aMayuscula(linea[i]);
		    }
		    //Mostrar lnea normal
		    //Buscamos instruccin
		    unsigned int limite=0;
		    while(limite <lineaSE.size() && isalpha(lineaSE[limite]))
		       limite++;
		    //Mostramos antes de busqueda
	            pagina << linea.substr(0,posReal[0]);
		    unsigned int mayor=0;
		    for(unsigned int i=2; i<=limite; i++)
		      if(reservadas.find(lineaSE.substr(0,i))!= reservadas.end())
		        mayor=i;
		    if(mayor>0) {
		       pagina << colorReservada <<
		       linea.substr(posReal[0],posReal[mayor]-posReal[0])
		       << finColorReservada;
		    }
		    else
		    {
		      for(unsigned int i=2; i<=limite; i++)
		        if(declaraciones.find(lineaSE.substr(0,i)) != declaraciones.end())
		          mayor=i;
		      if(mayor>0) {
		         pagina << colorVariable <<
		         linea.substr(posReal[0],posReal[mayor]-posReal[0])
		         << finColorVariable;
		      }
		    }
		    //Resto ristras y comentarios
		    if(posReal[mayor] < linea.size())
	  	       muestraRestoLinea(pagina,linea.substr(posReal[mayor],linea.size()-posReal[mayor]));
		 }
	      }
	   }

	void MuestraCodigoFortran77::muestraRestoLinea(HTML &pagina,string resto)
	{
	   char anterior='\0';
	   string actuals=" ";
	   if(enRistra) pagina << colorRistra;
	   for(unsigned int i=0; i<resto.size(); anterior=resto[i], i++)
	   {
	     char actual=resto[i];
	     actuals[0]=actual;
	     if(enRistra){
	       pagina << actuals;
	       if(actual == '\'' && anterior != '\\') //Fin de ristra
	       {
	         enRistra=false;
		 pagina << finColorRistra;
	       }
	     }
	     else{
	       if(actual == '!'){
	         pagina << colorComentario;
		 pagina << resto.substr(i,resto.size()-i);
		 pagina << finColorComentario;
		 return;
	       }
	       if(actual == '\''){
	         enRistra=true;
		 pagina << colorRistra;
	       }
	       pagina << actuals;
	     }
	   }
	}
*/

	public String getType(){
		return "fortran77";
	}
	public String getTypeName(){
		return "Fortran 77";
	}

}
