/**
 * $Id: File.java,v 1.6 2009/07/28 22:55:08 juanca Exp $
 * gui.* is part of ACodeEditor
 * Copyright (C) 2009 Juan Carlos Rodrguez-del-Pino. All rights reserved.
 * license GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author Juan Carlos Rodriguez-del-Pino
 **/
package gui;

import java.awt.Dimension;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

import editors.CodeEditorFactory;
import editors.GenericCodeEditor;
import editors.Highlights;
import editors.LineNumber;
import editors.StyledEditorKitHS;

public class File{
	private String name;
	private JScrollPane scroll;
	private JCodeEditorPane editor;
	private JSplitPane sp;
	private LineNumber lineNumber;
	private Highlights highlights;
	public File(String name, String code, Highlights highlights,LineNumber lineNumber,
				Main mainWindow, boolean showLineNumber){
		this.highlights = highlights;
		this.name=name;
		this.lineNumber=lineNumber;
		editor = new JCodeEditorPane();
		editor.setEditorKitForContentType("text/code", new StyledEditorKitHS());
		editor.setContentType("text/code");
		editor.setFont(highlights.getRegularFont());
		editor.setComponentPopupMenu(mainWindow.getPopupMenu());
		GenericCodeEditor doc=CodeEditorFactory.getCodigo(name,highlights);
		doc.setPane(editor);
		editor.setDocument(doc);
		try {
			doc.setText(code);
		} catch (BadLocationException e) {}
		doc.initUndo();
		sp= new JSplitPane();
		sp.setBorder(null);
		if(showLineNumber)
		   sp.setLeftComponent(lineNumber);
		else
		   sp.setLeftComponent(null);
		sp.setRightComponent(editor);
		sp.setDividerSize(0);
		scroll = new JScrollPane(sp);
		scroll.getVerticalScrollBar().setUnitIncrement(highlights.getCharHeight());
	}
	public void changeFontSize(Highlights highlights){
		this.highlights = highlights;
		GenericCodeEditor code=(GenericCodeEditor)editor.getDocument();
		code.setParagraphAttributes(0, code.getLength(), highlights.getRegular(), true);
		code.changed(0);
		scroll.getVerticalScrollBar().setUnitIncrement(highlights.getCharHeight());			
	}
	public void setPreferedSize(Dimension d){
		scroll.setPreferredSize(d);
	}
	public String getText(){
		Document doc=editor.getDocument();
		try {
			return doc.getText(0, doc.getLength());
		} catch (BadLocationException e) {}
		return "";
	}
	public String getFileName(){
		return name;
	}
	public void changeFileName(String newName){
		name=newName;
		GenericCodeEditor doc=CodeEditorFactory.getCodigo(name,highlights);
		doc.setPane(editor);
		GenericCodeEditor previous=(GenericCodeEditor)editor.getDocument();
		previous.dataTransfer(doc);
		int pos=editor.getCaretPosition();
		editor.setDocument(doc);
		editor.setCaretPosition(pos);
		previous.setStop();
	}
	public JComponent getPane(){
		return scroll;
	}
	public JCodeEditorPane getEditor(){
		return editor;
	}
	public boolean isNumerated() {
		return sp.getLeftComponent() != null;
	}
	public void setNumeration() {
		sp.setLeftComponent(lineNumber);
	}
	public void removeNumeration() {
		sp.setLeftComponent(null);
	}
}

